@@ -7,11 +7,12 @@ use crate::impl_const::*;
 
 use core::convert::TryFrom;
 use core::fmt::Display;
+use core::iter::FromIterator;
 use core::num::TryFromIntError;
 use core::ops::RangeInclusive;
-#[cfg(feature = "serde")]
-use serde::{Deserialize, Serialize};
-use yoke::{Yokeable, ZeroCopyFrom};
+use icu_uniset::UnicodeSet;
+use yoke::Yokeable;
+use zerofrom::ZeroFrom;
 use zerovec::ZeroVec;
 use zerovec::ZeroVecError;
 
@@ -19,7 +20,8 @@ use zerovec::ZeroVecError;
 /// would make it small or fast.
 /// See [`UCPTrieType`](https://unicode-org.github.io/icu-docs/apidoc/dev/icu4c/ucptrie_8h.html) in ICU4C.
 #[derive(Clone, Copy, PartialEq, Debug, Eq)]
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
+#[cfg_attr(feature = "serde_serialize", derive(serde::Serialize))]
 pub enum TrieType {
     /// Represents the "fast" type code point tries for the
     /// [`TrieType`] trait. The "fast max" limit is set to `0xffff`.
@@ -49,7 +51,7 @@ pub trait TrieValue: Copy + Eq + PartialEq + zerovec::ule::AsULE + 'static {
 }
 
 impl TrieValue for u8 {
-    const DATA_GET_ERROR_VALUE: u8 = u8::MAX;
+    const DATA_GET_ERROR_VALUE: u8 = 0;
     type TryFromU32Error = TryFromIntError;
     fn try_from_u32(i: u32) -> Result<Self, Self::TryFromU32Error> {
         Self::try_from(i)
@@ -57,7 +59,7 @@ impl TrieValue for u8 {
 }
 
 impl TrieValue for u16 {
-    const DATA_GET_ERROR_VALUE: u16 = u16::MAX;
+    const DATA_GET_ERROR_VALUE: u16 = 0;
     type TryFromU32Error = TryFromIntError;
     fn try_from_u32(i: u32) -> Result<Self, Self::TryFromU32Error> {
         Self::try_from(i)
@@ -65,7 +67,7 @@ impl TrieValue for u16 {
 }
 
 impl TrieValue for u32 {
-    const DATA_GET_ERROR_VALUE: u32 = u32::MAX;
+    const DATA_GET_ERROR_VALUE: u32 = 0;
     type TryFromU32Error = TryFromIntError;
     fn try_from_u32(i: u32) -> Result<Self, Self::TryFromU32Error> {
         Ok(i)
@@ -102,8 +104,9 @@ fn maybe_filter_value<T: TrieValue>(value: T, trie_null_value: T, null_value: T)
 /// For more information:
 /// - [ICU Site design doc](http://site.icu-project.org/design/struct/utrie)
 /// - [ICU User Guide section on Properties lookup](https://unicode-org.github.io/icu/userguide/strings/properties.html#lookup)
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug, Eq, PartialEq, Yokeable, ZeroCopyFrom)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
+#[cfg_attr(feature = "serde_serialize", derive(serde::Serialize))]
+#[derive(Debug, Eq, PartialEq, Yokeable, ZeroFrom)]
 pub struct CodePointTrie<'trie, T: TrieValue> {
     header: CodePointTrieHeader,
     #[cfg_attr(feature = "serde", serde(borrow))]
@@ -113,8 +116,9 @@ pub struct CodePointTrie<'trie, T: TrieValue> {
 }
 
 /// This struct contains the fixed-length header fields of a [`CodePointTrie`].
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Copy, Clone, Debug, Eq, PartialEq, Yokeable, ZeroCopyFrom)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
+#[cfg_attr(feature = "serde_serialize", derive(serde::Serialize))]
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Yokeable, ZeroFrom)]
 pub struct CodePointTrieHeader {
     /// The code point of the start of the last range of the trie. A
     /// range is defined as a partition of the code point space such that the
@@ -376,30 +380,6 @@ impl<'trie, T: TrieValue> CodePointTrie<'trie, T> {
             data: converted_data,
         })
     }
-}
-
-impl<'trie, T: TrieValue + Into<u32>> CodePointTrie<'trie, T> {
-    /// Returns the value that is associated with `code_point` for this [`CodePointTrie`]
-    /// as a `u32`.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use icu_codepointtrie::planes;
-    /// let trie = planes::get_planes_trie();
-    ///
-    /// let cp = '𑖎' as u32;
-    /// assert_eq!(cp, 0x1158E);
-    ///
-    /// let plane_num: u8 = trie.get(cp);
-    /// assert_eq!(trie.get_u32(cp), plane_num as u32);
-    /// ```
-    ///
-    // Note: This API method maintains consistency with the corresponding
-    // original ICU APIs.
-    pub fn get_u32(&self, code_point: u32) -> u32 {
-        self.get(code_point).into()
-    }
 
     /// Returns a [`CodePointMapRange`] struct which represents a range of code
     /// points associated with the same trie value. The returned range will be
@@ -656,9 +636,16 @@ impl<'trie, T: TrieValue + Into<u32>> CodePointTrie<'trie, T> {
                                 // as a transformation by applying maybe_filter_value()
                                 // to the trie value.
                                 // The current trie value `trie_value_2` within this data block
-                                // differs here from `trie_value`, and updating `trie_value`
-                                // with the new value may or may not help in computing the
-                                // range and/or yield a different result for maybe_filter_range().
+                                // differs here from the previous value in `trie_value`.
+                                // But both map to `value` after applying `maybe_filter_value`.
+                                // It is not clear whether the previous or the current trie value
+                                // (or neither) is more likely to match potential subsequent trie
+                                // values that would extend the range by mapping to `value`.
+                                // On the assumption of locality -- often times consecutive
+                                // characters map to the same trie values -- remembering the new
+                                // one might make it faster to extend this range further
+                                // (by increasing the chance that the next `trie_value_2 !=
+                                // trie_value` test will be false).
                                 trie_value = trie_value_2; // may or may not help
                             }
                         } else {
@@ -694,9 +681,16 @@ impl<'trie, T: TrieValue + Into<u32>> CodePointTrie<'trie, T> {
                                 // as a transformation by applying maybe_filter_value()
                                 // to the trie value.
                                 // The current trie value `trie_value_2` within this data block
-                                // differs here from `trie_value`, and updating `trie_value`
-                                // with the new value may or may not help in computing the
-                                // range and/or yield a different result for maybe_filter_range().
+                                // differs here from the previous value in `trie_value`.
+                                // But both map to `value` after applying `maybe_filter_value`.
+                                // It is not clear whether the previous or the current trie value
+                                // (or neither) is more likely to match potential subsequent trie
+                                // values that would extend the range by mapping to `value`.
+                                // On the assumption of locality -- often times consecutive
+                                // characters map to the same trie values -- remembering the new
+                                // one might make it faster to extend this range further
+                                // (by increasing the chance that the next `trie_value_2 !=
+                                // trie_value` test will be false).
                                 trie_value = trie_value_2; // may or may not help
                             }
 
@@ -782,6 +776,83 @@ impl<'trie, T: TrieValue + Into<u32>> CodePointTrie<'trie, T> {
             cpm_range: init_range,
         }
     }
+
+    /// Yields an [`Iterator`] returning the ranges of the code points whose values
+    /// match `value` in the [`CodePointTrie`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu_codepointtrie::planes;
+    ///
+    /// let trie = planes::get_planes_trie();
+    ///
+    /// let plane_val = 2;
+    /// let mut sip_range_iter = trie.get_ranges_for_value(plane_val as u8);
+    ///
+    /// let start = plane_val * 0x1_0000;
+    /// let end = start + 0xffff;
+    ///
+    /// let sip_range = sip_range_iter.next()
+    ///     .expect("Plane 2 (SIP) should exist in planes data");
+    /// assert_eq!(start..=end, sip_range);
+    ///
+    /// assert!(sip_range_iter.next().is_none());
+    pub fn get_ranges_for_value(&self, value: T) -> impl Iterator<Item = RangeInclusive<u32>> + '_ {
+        self.iter_ranges()
+            .filter(move |cpm_range| cpm_range.value == value)
+            .map(|cpm_range| cpm_range.range)
+    }
+
+    /// Returns a [`UnicodeSet`] for the code points that have the given
+    /// [`TrieValue`] in the trie.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu_codepointtrie::planes;
+    ///
+    /// let trie = planes::get_planes_trie();
+    ///
+    /// let plane_val = 2;
+    /// let sip = trie.get_set_for_value(plane_val as u8);
+    ///
+    /// let start = plane_val * 0x1_0000;
+    /// let end = start + 0xffff;
+    ///
+    /// assert!(!sip.contains_u32(start - 1));
+    /// assert!(sip.contains_u32(start));
+    /// assert!(sip.contains_u32(end));
+    /// assert!(!sip.contains_u32(end + 1));
+    /// ```
+    pub fn get_set_for_value(&self, value: T) -> UnicodeSet<'static> {
+        let value_ranges = self.get_ranges_for_value(value);
+        UnicodeSet::from_iter(value_ranges)
+    }
+}
+
+impl<'trie, T: TrieValue + Into<u32>> CodePointTrie<'trie, T> {
+    /// Returns the value that is associated with `code_point` for this [`CodePointTrie`]
+    /// as a `u32`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu_codepointtrie::planes;
+    /// let trie = planes::get_planes_trie();
+    ///
+    /// let cp = '𑖎' as u32;
+    /// assert_eq!(cp, 0x1158E);
+    ///
+    /// let plane_num: u8 = trie.get(cp);
+    /// assert_eq!(trie.get_u32(cp), plane_num as u32);
+    /// ```
+    ///
+    // Note: This API method maintains consistency with the corresponding
+    // original ICU APIs.
+    pub fn get_u32(&self, code_point: u32) -> u32 {
+        self.get(code_point).into()
+    }
 }
 
 impl<'trie, T: TrieValue> Clone for CodePointTrie<'trie, T>
@@ -814,7 +885,7 @@ pub struct CodePointMapRangeIterator<'a, T: TrieValue> {
     cpm_range: Option<CodePointMapRange<T>>,
 }
 
-impl<'a, T: TrieValue + Into<u32>> Iterator for CodePointMapRangeIterator<'a, T> {
+impl<'a, T: TrieValue> Iterator for CodePointMapRangeIterator<'a, T> {
     type Item = CodePointMapRange<T>;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -838,11 +909,11 @@ mod tests {
     use super::*;
     use crate::planes;
     use alloc::vec::Vec;
-    #[cfg(feature = "serde")]
+    #[cfg(feature = "serde_serialize")]
     use zerovec::ZeroVec;
 
     #[test]
-    #[cfg(feature = "serde")]
+    #[cfg(feature = "serde_serialize")]
     fn test_serde_with_postcard_roundtrip() -> Result<(), postcard::Error> {
         let trie = crate::planes::get_planes_trie();
         let trie_serialized: Vec<u8> = postcard::to_allocvec(&trie).unwrap();
